# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

# IBM starts
from cloudinit.settings import PER_ALWAYS
# IBM ends
from cloudinit import util
from cloudinit import netinfo
import socket

# IBM starts
frequency = PER_ALWAYS
# IBM ends

# IBM start - RTC 75283
# This class is added here and called in cmd/main.py
# Blocking set_hostname.py
class SetHostnameError(Exception):
    """Raised when the distro runs into an exception when setting hostname.
    This may happen if we attempt to set the hostname early in cloud-init's
    init-local timeframe as certain services may not be running yet."""
    pass
# IBM ends


def handle(name, _cfg, _cloud, log, _args):
    default_interface = 'eth0'
    system_info = util.system_info()
    if 'aix' in system_info['platform'].lower():
        default_interface = 'en0'

    interface = util.get_cfg_option_str(_cfg,
                                        'set_hostname_from_interface',
                                        default=default_interface)
    log.debug('Setting hostname based on interface %s' % interface)
    set_hostname = False
    fqdn = None
    # Look up the IP address on the interface
    # and then reverse lookup the hostname in DNS
    info = netinfo.netdev_info()
    if interface in info:
        set_short = util.get_cfg_option_bool(_cfg, "set_dns_shortname", False)
        #IBM start - RTC 74462
        if 'ipv4' in info[interface] and info[interface].get('ipv4'):
            # Handle IPv4 address
            interface_detail = info[interface]
            for interface in interface_detail['ipv4']:
                try:
                    ip = interface['ip']
                except:
                    log.info('Cannot find the IP')
            set_hostname =_set_hostname(_cfg, _cloud, log,
                                        ip, set_short)
        elif 'ipv6' in info[interface] and info[interface].get('ipv6'):
            # Handle IPv6 addresses
            addr = info[interface]
            for ipaddr in addr['ipv6']:
                try:
                    ip = ipaddr['ip']
                except:
                    log.info('Cannot find the IP')
                set_hostname =_set_hostname(_cfg, _cloud, log,
                                        ip, set_short)
                if set_hostname:
                    break
        # IBM ends
    else:
        log.warning('Interface %s was not found on the system. '
                    'Interfaces found on system: %s' % (interface,
                                                        info.keys()))

    # Reverse lookup failed, fall back to cc_set_hostname way.
    if not set_hostname:
        (short_hostname, fqdn) = util.get_hostname_fqdn(_cfg, _cloud)
        try:
            log.info('Fall back to setting hostname on VM as %s' % fqdn)
            _cloud.distro.set_hostname(short_hostname, fqdn=fqdn)
        except Exception:
            util.logexc(log, "Failed to set the hostname to %s", fqdn)
            raise


def _set_hostname(_cfg, _cloud, log, ipaddr, set_short):
    log.debug('ipaddr: %s' % ipaddr)
    try:
        addrinfo = socket.getaddrinfo(ipaddr, None, 0, socket.SOCK_STREAM)
        log.debug('addrinfo: %s' % addrinfo)
        if addrinfo:
            (fqdn, port) = socket.getnameinfo(addrinfo[0][4],
                                              socket.NI_NAMEREQD)
            if fqdn:
                log.info('Setting hostname on VM as %s' % fqdn)
                hostname = fqdn.split('.')[0] if set_short else fqdn
                _cloud.distro.set_hostname(hostname, fqdn=hostname)
                return True
    except socket.error:
        log.warning('No hostname found for IP address %s' % ipaddr)
    except socket.gaierror:
        log.warning('Unable to resolve hostname for IP address %s' % ipaddr)
    return False
